@@ -1,4 +1,4 @@
-/*  $Id: search_strategy.cpp 391263 2013-03-06 18:02:05Z rafanovi $
+/*  $Id: search_strategy.cpp 481634 2015-10-14 15:21:43Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -59,8 +59,10 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 BEGIN_SCOPE(blast)
 
-CImportStrategy::CImportStrategy(CRef<objects::CBlast4_request> request)
- : m_Request(request)
+    CImportStrategy::CImportStrategy(CRef<objects::CBlast4_request> request,
+                                     bool ignore_unsupported_options)
+    : m_Request(request),
+      m_IgnoreUnsupportedOptions(ignore_unsupported_options)
 {
 
     if (m_Request.Empty()) {
@@ -82,7 +84,8 @@ CImportStrategy::FetchData()
     const CBlast4_queue_search_request& req(m_Request->GetBody().GetQueue_search());
     m_OptionsBuilder.reset(new CBlastOptionsBuilder(req.GetProgram(),
                                                     req.GetService(),
-                                  CBlastOptions::eBoth));
+                                                    CBlastOptions::eBoth,
+                                                    m_IgnoreUnsupportedOptions));
 
     // Create the BLAST options
     const CBlast4_parameters* algo_opts(0);
@@ -116,7 +119,12 @@ CImportStrategy::FetchData()
         m_OptionsBuilder->GetSearchOptions(algo_opts, prog_opts, format_opts,
                                            &m_Data->m_Task);
     m_Data->m_QueryRange = m_OptionsBuilder->GetRestrictedQueryRange();
-    m_Data->m_FilteringID = m_OptionsBuilder->GetDbFilteringAlgorithmId();
+    if(m_OptionsBuilder->HasDbFilteringAlgorithmId())
+    	m_Data->m_FilteringID = m_OptionsBuilder->GetDbFilteringAlgorithmId();
+    if(m_OptionsBuilder->HasDbFilteringAlgorithmKey())
+    	m_Data->m_FilteringKey = m_OptionsBuilder->GetDbFilteringAlgorithmKey();
+    if(m_OptionsBuilder->GetSubjectMaskingType())
+    	m_Data->m_SubjectMaskingType = m_OptionsBuilder->GetSubjectMaskingType();
     m_Data->valid = true;
 }
 
@@ -177,6 +185,26 @@ CImportStrategy::GetDBFilteringID()
     return m_Data->m_FilteringID;
 }
 
+string
+CImportStrategy::GetDBFilteringKey()
+{
+    if (!m_Data->valid)
+           FetchData();
+
+    return m_Data->m_FilteringKey;
+}
+
+
+/// Get Subject Masking Type
+ESubjectMaskingType
+CImportStrategy::GetSubjectMaskingType()
+{
+    if (!m_Data->valid)
+           FetchData();
+
+    return m_Data->m_SubjectMaskingType;
+}
+
 string 
 CImportStrategy::GetService() const
 {
@@ -378,22 +406,51 @@ void CExportStrategy::x_Process_SearchDb(CRef<CSearchDatabase> & db)
     const CSearchDatabase::TGiList& gi_list_limit = db->GetGiListLimitation();
     if (!gi_list_limit.empty())
     {
-    	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_GiList), gi_list_limit);
+		#ifdef NCBI_STRICT_GI
+       		vector<int> int_gi_list;
+       		ITERATE ( vector<TGi>, it, gi_list_limit ) {
+       			int_gi_list.push_back(GI_TO( int, *it));
+       		}
+       		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_GiList), int_gi_list);
+		#else
+    		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_GiList), gi_list_limit);
+		#endif
     }
 
     // Set the negative GI list
     const CSearchDatabase::TGiList& neg_gi_list = db->GetNegativeGiListLimitation();
     if (!neg_gi_list.empty())
     {
-    	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeGiList), neg_gi_list);
+		#ifdef NCBI_STRICT_GI
+       		vector<int> int_gi_list;
+       		ITERATE ( vector<TGi>, it, neg_gi_list) {
+       			int_gi_list.push_back(GI_TO( int, *it));
+       		}
+       		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeGiList), int_gi_list);
+		#else
+       		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeGiList), neg_gi_list);
+		#endif
     }
 
     // Set the filtering algorithms
-    int algo_id = db->GetFilteringAlgorithm();
-    if (algo_id != -1)
+    string algo_key = db->GetFilteringAlgorithmKey();
+    if (algo_key != kEmptyStr)
     {
-       	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_DbFilteringAlgorithmId), algo_id);
+    	int mask_type = (int) db->GetMaskType();
+       	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_DbFilteringAlgorithmKey), algo_key);
+       	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_SubjectMaskingType), mask_type);
     }
+    else {
+    	// Set the filtering algorithms
+    	int algo_id = db->GetFilteringAlgorithm();
+    	if (algo_id != -1)
+    	{
+    		int mask_type = (int) db->GetMaskType();
+    		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_DbFilteringAlgorithmId), algo_id);
+    		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_SubjectMaskingType), mask_type);
+    	}
+    }
+
 }
 
 /* Prerequisite for calling x_Process_Pssm:-
@@ -588,6 +645,38 @@ void CExportStrategy::x_AddParameterToProgramOptions(objects::CBlast4Field & fie
     m_QueueSearchRequest->SetProgram_options().Set().push_back(p);
 }
 
+void CExportStrategy::x_AddParameterToProgramOptions(objects::CBlast4Field & field,
+                                 	 	    		 const vector<Int8> & int_list)
+{
+	list<Int8> tmp_list;
+    copy(int_list.begin(), int_list.end(), back_inserter(tmp_list));
+
+    CRef<CBlast4_parameter> p(new CBlast4_parameter);
+    p->SetName(field.GetName());
+
+    CRef<CBlast4_value> v(new CBlast4_value);
+    v->SetBig_integer_list() = tmp_list;
+    p->SetValue(*v);
+    _ASSERT(field.Match(*p));
+
+    m_QueueSearchRequest->SetProgram_options().Set().push_back(p);
+}
+
+
+void CExportStrategy::x_AddParameterToProgramOptions(objects::CBlast4Field & field,
+                                 	 	    		 const string & str)
+{
+	CRef<CBlast4_parameter> p(new CBlast4_parameter);
+	p->SetName(field.GetName());
+
+	CRef<CBlast4_value> v(new CBlast4_value);
+	v->SetString(str);
+	p->SetValue(*v);
+	_ASSERT(field.Match(*p));
+
+	m_QueueSearchRequest->SetProgram_options().Set().push_back(p);
+}
+
 void CExportStrategy::x_AddPsiNumOfIterationsToFormatOptions(unsigned int num_iters)
 {
 	CRef<CBlast4_parameter> p(new CBlast4_parameter);
